@@ -48,11 +48,6 @@ in this Software without prior written authorization from The Open Group.
 #include <X11/keysymdef.h>
 #include <stdio.h>
 
-#ifdef USE_OWN_COMPOSE
-#include "imComp.h"
-
-#endif
-
 #include "Xresource.h"
 #include "Key.h"
 
@@ -239,7 +234,7 @@ XRefreshKeyboardMapping(register XMappingEvent *event)
 	 */
 	LockDisplay(event->display);
 	if (event->display->keysyms) {
-	     Xfree ((char *)event->display->keysyms);
+	     Xfree (event->display->keysyms);
 	     event->display->keysyms = NULL;
 	}
 	UnlockDisplay(event->display);
@@ -276,12 +271,13 @@ _XKeyInitialize(
 	if (! keysyms) return 0;
 
 	LockDisplay(dpy);
-	if (dpy->keysyms)
-	    Xfree ((char *)dpy->keysyms);
+
+	Xfree (dpy->keysyms);
 	dpy->keysyms = keysyms;
 	dpy->keysyms_per_keycode = per;
 	if (dpy->modifiermap)
 	    ResetModMap(dpy);
+
 	UnlockDisplay(dpy);
     }
     if (!dpy->modifiermap)
@@ -294,7 +290,7 @@ UCSConvertCase( register unsigned code,
                 KeySym *lower,
                 KeySym *upper )
 {
-    /* Case conversion for UCS, as in Unicode Data version 4.0.0 */
+    /* Case conversion for UCS, as in Unicode Data version 4.0.0. */
     /* NB: Only converts simple one-to-one mappings. */
 
     /* Tables are used where they take less space than     */
@@ -339,9 +335,9 @@ UCSConvertCase( register unsigned code,
     0x01C9, 0x01C9, 0x01CC, 0x01CC, 0x01CC
     };
 
-    static unsigned short const Greek_upper_mapping[] = {
-    0x0000, 0x0000, 0x0000, 0x0000, 0x0374, 0x0375, 0x0000, 0x0000,
-    0x0000, 0x0000, 0x037A, 0x0000, 0x0000, 0x0000, 0x037E, 0x0000,
+    static unsigned short const Greek_upper_mapping[] = { /* updated to UD 14.0 */
+    0x0370, 0x0370, 0x0372, 0x0372, 0x0374, 0x0375, 0x0376, 0x0376,
+    0x0000, 0x0000, 0x037A, 0x03FD, 0x03FE, 0x03FF, 0x037E, 0x037F,
     0x0000, 0x0000, 0x0000, 0x0000, 0x0384, 0x0385, 0x0386, 0x0387,
     0x0388, 0x0389, 0x038A, 0x0000, 0x038C, 0x0000, 0x038E, 0x038F,
     0x0390, 0x0391, 0x0392, 0x0393, 0x0394, 0x0395, 0x0396, 0x0397,
@@ -351,18 +347,18 @@ UCSConvertCase( register unsigned code,
     0x03B0, 0x0391, 0x0392, 0x0393, 0x0394, 0x0395, 0x0396, 0x0397,
     0x0398, 0x0399, 0x039A, 0x039B, 0x039C, 0x039D, 0x039E, 0x039F,
     0x03A0, 0x03A1, 0x03A3, 0x03A3, 0x03A4, 0x03A5, 0x03A6, 0x03A7,
-    0x03A8, 0x03A9, 0x03AA, 0x03AB, 0x038C, 0x038E, 0x038F, 0x0000,
-    0x0392, 0x0398, 0x03D2, 0x03D3, 0x03D4, 0x03A6, 0x03A0, 0x03D7,
+    0x03A8, 0x03A9, 0x03AA, 0x03AB, 0x038C, 0x038E, 0x038F, 0x03CF,
+    0x0392, 0x0398, 0x03D2, 0x03D3, 0x03D4, 0x03A6, 0x03A0, 0x03CF,
     0x03D8, 0x03D8, 0x03DA, 0x03DA, 0x03DC, 0x03DC, 0x03DE, 0x03DE,
     0x03E0, 0x03E0, 0x03E2, 0x03E2, 0x03E4, 0x03E4, 0x03E6, 0x03E6,
     0x03E8, 0x03E8, 0x03EA, 0x03EA, 0x03EC, 0x03EC, 0x03EE, 0x03EE,
-    0x039A, 0x03A1, 0x03F9, 0x03F3, 0x03F4, 0x0395, 0x03F6, 0x03F7,
-    0x03F7, 0x03F9, 0x03FA, 0x03FA, 0x0000, 0x0000, 0x0000, 0x0000
+    0x039A, 0x03A1, 0x03F9, 0x037F, 0x03F4, 0x0395, 0x03F6, 0x03F7,
+    0x03F7, 0x03F9, 0x03FA, 0x03FA, 0x03FC, 0x03FD, 0x03FE, 0x03FF
     };
 
-    static unsigned short const Greek_lower_mapping[] = {
-    0x0000, 0x0000, 0x0000, 0x0000, 0x0374, 0x0375, 0x0000, 0x0000,
-    0x0000, 0x0000, 0x037A, 0x0000, 0x0000, 0x0000, 0x037E, 0x0000,
+    static unsigned short const Greek_lower_mapping[] = { /* updated to UD 14.0 */
+    0x0371, 0x0371, 0x0373, 0x0373, 0x0374, 0x0375, 0x0377, 0x0377,
+    0x0000, 0x0000, 0x037A, 0x037B, 0x037C, 0x037D, 0x037E, 0x03F3,
     0x0000, 0x0000, 0x0000, 0x0000, 0x0384, 0x0385, 0x03AC, 0x0387,
     0x03AD, 0x03AE, 0x03AF, 0x0000, 0x03CC, 0x0000, 0x03CD, 0x03CE,
     0x0390, 0x03B1, 0x03B2, 0x03B3, 0x03B4, 0x03B5, 0x03B6, 0x03B7,
@@ -372,13 +368,13 @@ UCSConvertCase( register unsigned code,
     0x03B0, 0x03B1, 0x03B2, 0x03B3, 0x03B4, 0x03B5, 0x03B6, 0x03B7,
     0x03B8, 0x03B9, 0x03BA, 0x03BB, 0x03BC, 0x03BD, 0x03BE, 0x03BF,
     0x03C0, 0x03C1, 0x03C2, 0x03C3, 0x03C4, 0x03C5, 0x03C6, 0x03C7,
-    0x03C8, 0x03C9, 0x03CA, 0x03CB, 0x03CC, 0x03CD, 0x03CE, 0x0000,
+    0x03C8, 0x03C9, 0x03CA, 0x03CB, 0x03CC, 0x03CD, 0x03CE, 0x03D7,
     0x03D0, 0x03D1, 0x03D2, 0x03D3, 0x03D4, 0x03D5, 0x03D6, 0x03D7,
     0x03D9, 0x03D9, 0x03DB, 0x03DB, 0x03DD, 0x03DD, 0x03DF, 0x03DF,
     0x03E1, 0x03E1, 0x03E3, 0x03E3, 0x03E5, 0x03E5, 0x03E7, 0x03E7,
     0x03E9, 0x03E9, 0x03EB, 0x03EB, 0x03ED, 0x03ED, 0x03EF, 0x03EF,
     0x03F0, 0x03F1, 0x03F2, 0x03F3, 0x03B8, 0x03F5, 0x03F6, 0x03F8,
-    0x03F8, 0x03F2, 0x03FB, 0x03FB, 0x0000, 0x0000, 0x0000, 0x0000
+    0x03F8, 0x03F2, 0x03FB, 0x03FB, 0x03FC, 0x037B, 0x037C, 0x037D
     };
 
     static unsigned short const GreekExt_lower_mapping[] = {
@@ -470,11 +466,13 @@ UCSConvertCase( register unsigned code,
             *upper = 0x0178;
         else if (code == 0x00b5)      /* micro sign */
             *upper = 0x039c;
+        else if (code == 0x00df)      /* ssharp */
+            *upper = 0x1e9e;
 	return;
     }
 
     /* Latin Extended-A, U+0100 to U+017F */
-    if (code >= 0x0100 && code <= 0x017f) {
+    if (code <= 0x017f) {
         if ( (code >= 0x0100 && code <= 0x012f) ||
              (code >= 0x0132 && code <= 0x0137) ||
              (code >= 0x014a && code <= 0x0177) ) {
@@ -500,13 +498,19 @@ UCSConvertCase( register unsigned code,
     }
 
     /* Latin Extended-B, U+0180 to U+024F */
-    if (code >= 0x0180 && code <= 0x024f) {
-        if (code >= 0x01cd && code <= 0x01dc) {
+    if (code <= 0x024f) {
+        if (code >= 0x0180 && code <= 0x01cc) {
+            *lower = LatinExtB_lower_mapping[code - 0x0180];
+            *upper = LatinExtB_upper_mapping[code - 0x0180];
+        }
+        else if (code >= 0x01cd && code <= 0x01dc) {
 	    if (code & 1)
 	       *lower += 1;
 	    else
 	       *upper -= 1;
         }
+        else if (code == 0x01dd)
+            *upper = 0x018e;
         else if ( (code >= 0x01de && code <= 0x01ef) ||
                   (code >= 0x01f4 && code <= 0x01f5) ||
                   (code >= 0x01f8 && code <= 0x021f) ||
@@ -514,12 +518,6 @@ UCSConvertCase( register unsigned code,
             *lower |= 1;
             *upper &= ~1;
         }
-        else if (code >= 0x0180 && code <= 0x01cc) {
-            *lower = LatinExtB_lower_mapping[code - 0x0180];
-            *upper = LatinExtB_upper_mapping[code - 0x0180];
-        }
-        else if (code == 0x01dd)
-            *upper = 0x018e;
         else if (code == 0x01f1 || code == 0x01f2) {
             *lower = 0x01f3;
             *upper = 0x01f1;
@@ -538,11 +536,13 @@ UCSConvertCase( register unsigned code,
     /* IPA Extensions, U+0250 to U+02AF */
     if (code >= 0x0253 && code <= 0x0292) {
         *upper = IPAExt_upper_mapping[code - 0x0253];
+        return;
     }
 
     /* Combining Diacritical Marks, U+0300 to U+036F */
     if (code == 0x0345) {
         *upper = 0x0399;
+        return;
     }
 
     /* Greek and Coptic, U+0370 to U+03FF */
@@ -553,11 +553,11 @@ UCSConvertCase( register unsigned code,
             *upper = code;
         if (*lower == 0)
             *lower = code;
+        return;
     }
 
     /* Cyrillic and Cyrillic Supplementary, U+0400 to U+052F */
-    if ( (code >= 0x0400 && code <= 0x04ff) ||
-         (code >= 0x0500 && code <= 0x052f) ) {
+    if ( (code >= 0x0400 && code <= 0x052f) ) {
         if (code >= 0x0400 && code <= 0x040f)
             *lower += 0x50;
         else if (code >= 0x0410 && code <= 0x042f)
@@ -580,6 +580,7 @@ UCSConvertCase( register unsigned code,
 	    else
 	        *upper -= 1;
         }
+        return;
     }
 
     /* Armenian, U+0530 to U+058F */
@@ -588,6 +589,7 @@ UCSConvertCase( register unsigned code,
             *lower += 0x30;
         else if (code >=0x0561 && code <= 0x0586)
             *upper -= 0x30;
+        return;
     }
 
     /* Latin Extended Additional, U+1E00 to U+1EFF */
@@ -599,6 +601,9 @@ UCSConvertCase( register unsigned code,
         }
         else if (code == 0x1e9b)
             *upper = 0x1e60;
+        else if (code == 0x1e9e)
+            *lower = 0x00df;    /* ssharp */
+        return;
     }
 
     /* Greek Extended, U+1F00 to U+1FFF */
@@ -609,6 +614,7 @@ UCSConvertCase( register unsigned code,
             *upper = code;
         if (*lower == 0)
             *lower = code;
+        return;
     }
 
     /* Letterlike Symbols, U+2100 to U+214F */
@@ -741,8 +747,9 @@ XConvertCase(
 	    *upper -= (XK_Greek_alphaaccent - XK_Greek_ALPHAaccent);
 	else if (sym >= XK_Greek_ALPHA && sym <= XK_Greek_OMEGA)
 	    *lower += (XK_Greek_alpha - XK_Greek_ALPHA);
-	else if (sym >= XK_Greek_alpha && sym <= XK_Greek_omega &&
-		 sym != XK_Greek_finalsmallsigma)
+	else if (sym == XK_Greek_finalsmallsigma)
+	    *upper = XK_Greek_SIGMA;
+	else if (sym >= XK_Greek_alpha && sym <= XK_Greek_omega)
 	    *upper -= (XK_Greek_alpha - XK_Greek_ALPHA);
         break;
     case 0x13: /* Latin 9 */
@@ -836,7 +843,7 @@ _XTranslateKeySym(
 	if (((modifiers & AllMods) == p->state) && (symbol == p->key)) {
 	    length = p->len;
 	    if (length > nbytes) length = nbytes;
-	    memcpy (buffer, p->string, length);
+	    memcpy (buffer, p->string, (size_t) length);
 	    return length;
 	}
     }
@@ -891,73 +898,6 @@ XLookupString (
 		  &modifiers, &symbol))
 	return 0;
 
-#ifdef USE_OWN_COMPOSE
-    if ( status ) {
-	static int been_here= 0;
-	if ( !been_here ) {
-	    XimCompInitTables();
-	    been_here = 1;
-	}
-	if ( !XimCompLegalStatus(status) ) {
-	    status->compose_ptr = NULL;
-	    status->chars_matched = 0;
-	}
-	if ( ((status->chars_matched>0)&&(status->compose_ptr!=NULL)) ||
-		XimCompIsComposeKey(symbol,event->keycode,status) ) {
-	    XimCompRtrn rtrn;
-	    switch (XimCompProcessSym(status,symbol,&rtrn)) {
-		case XIM_COMP_IGNORE:
-		    break;
-		case XIM_COMP_IN_PROGRESS:
-		    if ( keysym!=NULL )
-			*keysym = NoSymbol;
-		    return 0;
-		case XIM_COMP_FAIL:
-		{
-		    int n = 0, len= 0;
-		    for (n=len=0;rtrn.sym[n]!=XK_VoidSymbol;n++) {
-			if ( nbytes-len > 0 ) {
-			    len+= _XTranslateKeySym(event->display,rtrn.sym[n],
-							event->state,
-							buffer+len,nbytes-len);
-			}
-		    }
-		    if ( keysym!=NULL ) {
-			if ( n==1 )	*keysym = rtrn.sym[0];
-			else		*keysym = NoSymbol;
-		    }
-		    return len;
-		}
-		case XIM_COMP_SUCCEED:
-		{
-		    int len,n = 0;
-
-		    symbol = rtrn.matchSym;
-		    if ( keysym!=NULL )	*keysym = symbol;
-		    if ( rtrn.str[0]!='\0' ) {
-			strncpy(buffer,rtrn.str,nbytes-1);
-			buffer[nbytes-1]= '\0';
-			len = strlen(buffer);
-		    }
-		    else {
-			len = _XTranslateKeySym(event->display,symbol,
-							event->state,
-							buffer,nbytes);
-		    }
-		    for (n=0;rtrn.sym[n]!=XK_VoidSymbol;n++) {
-			if ( nbytes-len > 0 ) {
-			    len+= _XTranslateKeySym(event->display,rtrn.sym[n],
-							event->state,
-							buffer+len,nbytes-len);
-			}
-		    }
-		    return len;
-		}
-	    }
-	}
-    }
-#endif
-
     if (keysym)
 	*keysym = symbol;
     /* arguable whether to use (event->state & ~modifiers) here */
@@ -974,9 +914,10 @@ _XFreeKeyBindings(
     for (p = dpy->key_bindings; p; p = np) {
 	np = p->next;
 	Xfree(p->string);
-	Xfree((char *)p->modifiers);
-	Xfree((char *)p);
+	Xfree(p->modifiers);
+	Xfree(p);
     }
+    dpy->key_bindings = NULL;
 }
 
 int
@@ -997,15 +938,13 @@ XRebindKeysym (
     tmp = dpy->key_bindings;
     nb = sizeof(KeySym) * nm;
 
-    if ((! (p = (struct _XKeytrans *) Xcalloc( 1, sizeof(struct _XKeytrans)))) ||
-	((! (p->string = (char *) Xmalloc( (unsigned) nbytes))) &&
-	 (nbytes > 0)) ||
-	((! (p->modifiers = (KeySym *) Xmalloc( (unsigned) nb))) &&
-	 (nb > 0))) {
+    if ((! (p = Xcalloc( 1, sizeof(struct _XKeytrans)))) ||
+	((! (p->string = Xmalloc(nbytes))) && (nbytes > 0)) ||
+	((! (p->modifiers = Xmalloc(nb))) && (nb > 0))) {
 	if (p) {
-	    if (p->string) Xfree(p->string);
-	    if (p->modifiers) Xfree((char *) p->modifiers);
-	    Xfree((char *) p);
+	    Xfree(p->string);
+	    Xfree(p->modifiers);
+	    Xfree(p);
 	}
 	UnlockDisplay(dpy);
 	return 0;
@@ -1014,13 +953,14 @@ XRebindKeysym (
     dpy->key_bindings = p;
     dpy->free_funcs->key_bindings = _XFreeKeyBindings;
     p->next = tmp;	/* chain onto list */
-    memcpy (p->string, (char *) str, nbytes);
+    memcpy (p->string, str, (size_t) nbytes);
     p->len = nbytes;
-    memcpy ((char *) p->modifiers, (char *) mlist, nb);
+    memcpy ((char *) p->modifiers, (char *) mlist, (size_t) nb);
     p->key = keysym;
     p->mlen = nm;
-    ComputeMaskFromKeytrans(dpy, p);
     UnlockDisplay(dpy);
+    ComputeMaskFromKeytrans(dpy, p);
+
     return 0;
 }
 
